using System;
using Autoscribe.Lyons.GeneralService;
using Autoscribe.Lyons.ATMService;
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace atmDemo.Wrapper
{
    public class ATMServiceWrapper : IATMServiceWrapper
    {
        public const string CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME = "ATMServiceSoap";
        public LogonResult Logon(int companyId, string username, string password)
        {
            LogonResult result = null;
            try
            {
                using (ChannelFactory<IATMService> fact = new ChannelFactory<IATMService>(CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME))
                {
                    IATMService svc = fact.CreateChannel();
                    result = svc.Logon(companyId, username, password);
                }
            }
            catch (FaultException fe)
            {
                MessageFault mf = fe.CreateMessageFault();
                string errorDetail = null;
                if (mf.HasDetail)
                {
                    ErrorResult errorData = mf.GetDetail<ErrorResult>();
                    errorDetail = errorData.ErrorMesssage;
                    Console.WriteLine(" --> Web Error: {0}", errorDetail);
                }
                result = new LogonResult(errorDetail);
            }
            catch (Exception ex)
            {
                Console.WriteLine(" --> Internal Error: {0}", ex.Message);
                result = new LogonResult("Internal Error");
            }            
            return result;
        }
        public void Logoff(string token)
        {
            try
            {
                using (ChannelFactory<IATMService> fact = new ChannelFactory<IATMService>(CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME))
                {
                    IATMService svc = fact.CreateChannel();
                    svc.Logoff(token);
                }
            }
            catch (FaultException fe)
            {
                MessageFault mf = fe.CreateMessageFault();
                string errorDetail = null;
                if (mf.HasDetail)
                {
                    ErrorResult errorData = mf.GetDetail<ErrorResult>();
                    errorDetail = errorData.ErrorMesssage;
                    Console.WriteLine(" --> Web Error: {0}", errorDetail);
                }                
            }
            catch (Exception ex)
            {
                Console.WriteLine(" --> Internal Error: {0}", ex.Message);
            }
        }
        public BooleanResult IsParticipant(string token, string aba)
        {
            BooleanResult result = null;
            try
            {
                using (ChannelFactory<IATMService> fact = new ChannelFactory<IATMService>(CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME))
                {
                    IATMService svc = fact.CreateChannel();
                    result = svc.IsParticipant(token, aba);
                }
            }
            catch (FaultException fe)
            {
                MessageFault mf = fe.CreateMessageFault();
                string errorDetail = null;
                if (mf.HasDetail)
                {
                    ErrorResult errorData = mf.GetDetail<BooleanResult>();
                    errorDetail = errorData.ErrorMesssage;
                    Console.WriteLine(" --> Web Error: {0}", errorDetail);
                }
                result = new BooleanResult(errorDetail);
            }
            catch (Exception ex)
            {
                Console.WriteLine(" --> Internal Error: {0}", ex.Message);
                result = new BooleanResult("Internal Error");
            }
            return result;
        }
        public StatusCodeResult VerifyAccount(string token, string aba, string account, string amount, string customerPresent)
        {
            StatusCodeResult result = null;
            try
            {
                using (ChannelFactory<IATMService> fact = new ChannelFactory<IATMService>(CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME))
                {
                    IATMService svc = fact.CreateChannel();
                    result = svc.VerifyAccount(token, aba, account, amount, customerPresent);
                }
            }
            catch (FaultException fe)
            {
                MessageFault mf = fe.CreateMessageFault();
                string errorDetail = null;
                if (mf.HasDetail)
                {
                    ErrorResult errorData = mf.GetDetail<ErrorResult>();
                    errorDetail = errorData.ErrorMesssage;
                    Console.WriteLine(" --> Web Error: {0}", errorDetail);
                }
                result = new StatusCodeResult(errorDetail);
            }
            catch (Exception ex)
            {
                Console.WriteLine(" --> Internal Error: {0}", ex.Message);
                result = new StatusCodeResult("Internal Error");
            }
            return result;
        }
        public StatusDescriptionResult DescribeStatusCode(string token, int statusCode)
        {
            StatusDescriptionResult result = null;
            try
            {
                using (ChannelFactory<IATMService> fact = new ChannelFactory<IATMService>(CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME))
                {
                    IATMService svc = fact.CreateChannel();
                    result = svc.DescribeStatusCode(token, statusCode);
                }
            }
            catch (FaultException fe)
            {
                MessageFault mf = fe.CreateMessageFault();
                string errorDetail = null;
                if (mf.HasDetail)
                {
                    ErrorResult errorData = mf.GetDetail<ErrorResult>();
                    errorDetail = errorData.ErrorMesssage;
                    Console.WriteLine(" --> Web Error: {0}", errorDetail);
                }
                result = new StatusDescriptionResult(errorDetail);
            }
            catch (Exception ex)
            {
                Console.WriteLine(" --> Internal Error: {0}", ex.Message);
                result = new StatusDescriptionResult("Internal Error");
            }
            return result;
        }
        public DetailedStatusCodeResult VerifyAccountWithDetails(string token, string aba, string account, string amount, string customerPresent, string denyNsf)
        {
            DetailedStatusCodeResult result = null;
            try
            {
                using (ChannelFactory<IATMService> fact = new ChannelFactory<IATMService>(CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME))
                {
                    IATMService svc = fact.CreateChannel();
                    result = svc.VerifyAccountWithDetails(token, aba, account, amount, customerPresent, denyNsf);
                }
            }
            catch (FaultException fe)
            {
                MessageFault mf = fe.CreateMessageFault();
                string errorDetail = null;
                if (mf.HasDetail)
                {
                    ErrorResult errorData = mf.GetDetail<ErrorResult>();
                    errorDetail = errorData.ErrorMesssage;
                    Console.WriteLine(" --> Web Error: {0}", errorDetail);
                }
                result = new DetailedStatusCodeResult(errorDetail);
            }
            catch (Exception ex)
            {
                Console.WriteLine(" --> Internal Error: {0}", ex.Message);
                result = new DetailedStatusCodeResult("Internal Error");
            }
            return result;
        }
    }
}
